using System;
using System.Drawing;
using System.IO;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Xml;
using AddinExpress.OL;
using Outlook = Microsoft.Office.Interop.Outlook;
using System.Xml.Linq;

namespace CustomOutlookProperties
{
    public partial class frmCustomProperties : AddinExpress.OL.ADXOlForm
    {
        public frmCustomProperties()
        {
            InitializeComponent();
        }

        private void frmCustomProperties_ADXAfterFormShow()
        {
            LoadAttachment();
            ShowProperties();
        }

        private void btnAdd_Click(object sender, EventArgs e)
        {
            txtPropertyName.Clear();
            txtPropertyValue.Clear();
            txtPropertyName.ReadOnly = false;
            panelAddNew.Visible = true;
        }

        private void btnEdit_Click(object sender, EventArgs e)
        {
            var selectedItem = lvProps.SelectedItems[0];
            if (selectedItem != null)
            {
                var valueItem = selectedItem.SubItems[1];
                txtPropertyName.Text = selectedItem.Text;
                txtPropertyName.ReadOnly = true;
                txtPropertyValue.Text = valueItem.Text;
                panelAddNew.Visible = true;
            }
        }

        private void btnDelete_Click(object sender, EventArgs e)
        {
            Outlook.Inspector currInsp = null;
            Outlook.MailItem mail = null;
            Outlook.UserProperties itemProps = null;

            try
            {
                currInsp = this.InspectorObj as Outlook.Inspector;
                if (currInsp.CurrentItem is Outlook.MailItem)
                {
                    mail = (Outlook.MailItem)currInsp.CurrentItem;
                    itemProps = mail.UserProperties;
                    foreach (var item in lvProps.SelectedItems)
                    {
                        ListViewItem selectedItem = item as ListViewItem;
                        lvProps.Items.Remove(selectedItem);

                        Outlook.UserProperty prop = itemProps.Find(selectedItem.Text);
                        if (prop != null)
                        {
                            prop.Delete();
                            Marshal.ReleaseComObject(prop);
                        }
                    }
                    mail.Save();
                }
            }
            finally
            {
                if (itemProps != null) Marshal.ReleaseComObject(itemProps);
                if (mail != null) Marshal.ReleaseComObject(mail);
            }
        }

        private void btnSave_Click(object sender, EventArgs e)
        {
            Outlook.Inspector currInsp = null;
            Outlook.MailItem mail = null;
            Outlook.UserProperties itemProps = null;
            Outlook.UserProperty newProp = null;

            try
            {
                currInsp = this.InspectorObj as Outlook.Inspector;
                if (currInsp.CurrentItem is Outlook.MailItem)
                {
                    mail = (Outlook.MailItem)currInsp.CurrentItem;
                    itemProps = mail.UserProperties;
                    newProp = itemProps.Find(txtPropertyName.Text);
                    if (newProp != null)
                    {
                        newProp.Value = txtPropertyValue.Text;
                    }
                    else
                    {
                        newProp = itemProps.Add(txtPropertyName.Text, 
                            Outlook.OlUserPropertyType.olText, true);
                    }

                    newProp.Value = txtPropertyValue.Text;
                    mail.Save();
                    panelAddNew.Visible = false;
                    ShowProperties();
                }
            }
            finally
            {
                if (newProp != null) Marshal.ReleaseComObject(newProp);
                if (itemProps != null) Marshal.ReleaseComObject(itemProps);
                if (mail != null) Marshal.ReleaseComObject(mail);
            }
        }

        private void LoadAttachment()
        {
            Outlook.Inspector currInsp = null;
            Outlook.MailItem mail = null;
            Outlook.UserProperties props = null;
            Outlook.Attachments attachments = null;
            string savePath = string.Format("{0}\\OutlookProperties\\userproperties.xml", 
                Environment.GetFolderPath(Environment.SpecialFolder.CommonApplicationData));

            try
            {
                currInsp = this.InspectorObj as Outlook.Inspector;
                if (currInsp.CurrentItem is Outlook.MailItem)
                {
                    mail = currInsp.CurrentItem as Outlook.MailItem;
                    attachments = mail.Attachments;
                    if (attachments.Count > 0)
                    {
                        props = mail.UserProperties;
                        for (int i = 1; i <= attachments.Count; i++)
                        {
                            Outlook.Attachment attachment = attachments[i];
                            if (attachment.FileName == "userproperties.xml")
                            {
                                attachment.SaveAsFile(savePath);
                                attachment.Delete();
                            }
                            Marshal.ReleaseComObject(attachment);
                        }
                        LoadProperties(props, savePath);
                        mail.Save();
                    }
                }
            }
            finally
            {
                if (mail != null) Marshal.ReleaseComObject(mail);
                if (attachments != null) Marshal.ReleaseComObject(attachments);
                if (props != null) Marshal.ReleaseComObject(props);
            }
        }

        private void ShowProperties()
        {
            Outlook.Inspector currInsp = null;
            Outlook.MailItem mail = null;
            Outlook.UserProperties itemProps = null;

            try
            {
                lvProps.Items.Clear();
                currInsp = this.InspectorObj as Outlook.Inspector;
                if (currInsp.CurrentItem is Outlook.MailItem)
                {
                    mail = (Outlook.MailItem)currInsp.CurrentItem;
                    itemProps = mail.UserProperties;

                    for (int i = 1; i <= itemProps.Count; i++)
                    {
                        Outlook.UserProperty property = itemProps[i];
                        ListViewItem item = new ListViewItem(property.Name);
                        ListViewItem.ListViewSubItem subItem = 
                            new ListViewItem.ListViewSubItem(item, property.Value.ToString());
                        item.SubItems.Add(subItem);
                        lvProps.Items.Add(item);
                        Marshal.ReleaseComObject(property);
                    }
                }
            }
            finally
            {
                if (itemProps != null) Marshal.ReleaseComObject(itemProps);
                if (mail != null) Marshal.ReleaseComObject(mail);
            }
        }

        private void LoadProperties(Outlook.UserProperties props, string filePath)
        {
            XDocument doc = null;
            doc = XDocument.Load(filePath);
            var xmlProperties = doc.Descendants("UserProperty");
            foreach (var xmlProperty in xmlProperties)
            {
                string value = xmlProperty.Value;
                string name = xmlProperty.Attribute("Name").Value;
                Outlook.UserProperty prop = props.Add(
                    xmlProperty.Attribute("Name").Value, 
                    Outlook.OlUserPropertyType.olText, 
                    true);
                prop.Value = xmlProperty.Value;
                Marshal.ReleaseComObject(prop);
            }
        }
    }
}
